@@ -1,7 +1,7 @@
 
 coord_desc <- function(start,func,type,grad,hess,hessFun,pars_pen,model,lambda,mats,
                        block,max.iter,tol,full,solver,solver.maxit,alpha.inc,momentum,step,
-                       step.ratio,diff_par){
+                       step.ratio,diff_par,pen_vec,e_alpha){
   count = 0
   ret <- list()
   max.iter = max.iter
@@ -46,9 +46,8 @@ coord_desc <- function(start,func,type,grad,hess,hessFun,pars_pen,model,lambda,m
       pen_diff=0
     }
 
-    #print(pen_diff)
-
       update.pars <- new.pars[count,]
+      #print(round(update.pars,4))
       #a.pars <- update.pars[1:max(mats$A)]
       #s.pars <- update.pars[min(mats$S != 0):max(mats$S)]
     # gg <- grad(new.pars[count,])
@@ -70,34 +69,45 @@ coord_desc <- function(start,func,type,grad,hess,hessFun,pars_pen,model,lambda,m
 
         if(full==TRUE & solver == FALSE){
           gg <- grad(new.pars[count,])
+          print(round(gg,3))
+          print(round(new.pars[count,],3))
 
-          print(func(new.pars[count,]))
-
+         # print(func(new.pars[count,]))
+          #update.pars2 <- new.pars[count,]
 
           update.pars <- update.pars - alpha*gg
 
-          if(type=="lasso" & lambda > 0){
+          if(type!="none" | type!="ridge" | type!="diff_lasso" & lambda > 0){
             for(j in pars_pen){
-              update.pars[j] <- sign(update.pars[j])*max(abs(update.pars[j])-alpha*lambda,0)
+              update.pars[j] <- soft(update.pars[j],lambda,type,step=alpha,e_alpha)
             }
           }
+
+
+
         }else if(full==TRUE & solver == TRUE){
           out <- nlminb(new.pars[count,],func,grad,control=list(iter.max=solver.maxit))
-          print(out$objective)
+         # print(out$objective)
           update.pars <- out$par
 
-          if(type=="lasso" & lambda > 0){
+          if(type!="none" | type!="ridge" | type!="diff_lasso" & lambda > 0){
             for(j in pars_pen){
-              update.pars[j] <- sign(update.pars[j])*max(abs(update.pars[j])-alpha*lambda,0)
+              update.pars[j] <- soft(update.pars[j],lambda,type,step=alpha,e_alpha)
             }
           }else if(type=="diff_lasso" & lambda > 0){
             for(j in pars_pen){
+              #print(update.pars[j])
               update.pars[j] <- update.pars[j] + sign(pen_diff[j])*max(abs(pen_diff[j])-alpha*lambda,0)
             }
+          }else if(type=="alasso" & lambda > 0){
+            for(j in pars_pen){
+              #print(update.pars[j])
+              update.pars[j] <- soft(pen_vec[j],lambda,type,step=alpha,e_alpha)
+            }
           }
 
         }else if(full==FALSE & line.search==FALSE){
-          # A
+
           gg <- grad(new.pars[count,])
 
           if(type=="diff_lasso"){
@@ -106,13 +116,14 @@ coord_desc <- function(start,func,type,grad,hess,hessFun,pars_pen,model,lambda,m
           update.pars[1:max(mats$A)] <- update.pars[1:max(mats$A)] - alpha1*t(gg[1:max(mats$A)])
 
 
-          if(type=="lasso" & lambda > 0){
+          if(type=="lasso" | type=="alasso" & lambda > 0){
             for(j in pars_pen){
-              update.pars[j] <- sign(update.pars[j])*max(abs(update.pars[j])-alpha1*lambda,0)
+              update.pars[j] <- soft(update.pars[j],lambda,type,step=alpha1,e_alpha)
             }
           }else if(type=="diff_lasso" & lambda > 0){
             for(j in 1:length(pen_diff)){
               ind <- pars_pen[j]
+              print(ind)
             #  print(round(sign(pen_diff[j])*max(abs(pen_diff[j])-alpha*lambda,0),3))
              # print(ind); print(sign(pen_diff[j])*max(abs(pen_diff[j])-alpha*lambda,0))
               #update.pars[j] <- sign(update.pars[j])*max(abs(update.pars[j])-alpha*lambda,0)
@@ -121,7 +132,7 @@ coord_desc <- function(start,func,type,grad,hess,hessFun,pars_pen,model,lambda,m
 
              # print(ind)
              # update.pars[ind] <- update.temp[j] - sign(pen_diff[j])*max(abs(pen_diff[j]) - alpha1*lambda,0)
-              pen  <- pen_diff[j] - sign(pen_diff[j])*max(abs(pen_diff[j]) - alpha1*lambda,0)
+              pen  <- pen_diff[j] - sign(pen_diff[j])*max(abs(pen_diff[j]) - alpha*lambda,0)
               update.pars[ind] <- update.temp[j] - pen
              # print(round(update.pars[ind],3));print(round(pen_diff[j],3))
              # update.pars[ind] <- update.temp[j] - pen_diff[j]
@@ -130,7 +141,11 @@ coord_desc <- function(start,func,type,grad,hess,hessFun,pars_pen,model,lambda,m
              # print(round(update.pars[ind],3))
 
             }
-          }
+          }#else if(type=="alasso" & lambda > 0){
+           # for(j in pars_pen){
+           #   update.pars[j] <- soft(pen_vec[j],lambda,type,step=alpha1,e_alpha)
+          #  }
+          #}
 
           ####################################################
           ####################################################
@@ -142,7 +157,7 @@ coord_desc <- function(start,func,type,grad,hess,hessFun,pars_pen,model,lambda,m
 
           # S
           gg2 <- grad(update.pars)
-          #print(rbind(t(gg),t(gg2)))
+         # print(round(rbind(t(gg),t(gg2))),4)
 
           update.pars[min(mats$S[mats$S !=0]):max(mats$S)] <-
             update.pars[min(mats$S[mats$S !=0]):max(mats$S)] - alpha2*gg2[min(mats$S[mats$S !=0]):max(mats$S)]
@@ -290,6 +305,8 @@ coord_desc <- function(start,func,type,grad,hess,hessFun,pars_pen,model,lambda,m
       convergence=99
     }else if(is.na(st.crit)==TRUE){
       convergence=99
+    }else if(any(new.pars[count+1,] > 500)){
+      convergence=99
     }else{
       if(st.crit==TRUE){
         convergence = 0
